﻿#ifndef ALARM_TREE_WIDGET_H
#define ALARM_TREE_WIDGET_H

#include "filter.h"
#include <QTreeWidget>
#include <vector>
#include <iostream>

class CuData;
class AlarmTreeWidgetItem;

using namespace std;

class AlarmTreeWidget : public QTreeWidget
{
    Q_OBJECT
public:
    enum Type {AlarmType, DisabledAlarmType };
    enum Columns { DATET, MICROSEC, ALARM, STATUS, ACK, LEVEL, SILENCED, DEVICE, MESSAGE, COLUMN_COUNT };
    enum SrvErrState { SrvErrNew = 0x1, SrvErrUnack = 0x2 };

    AlarmTreeWidget(QWidget *parent);
    void setType(Type t);
    int findPos(time_t ts, const QString& pri, const QString &status);
    int countAll() const;
    int countAlarmed() const;
    void setServerErrNotNew(const QString& errdev);
    void setServerErrAck(const QString& n);
    void deselectAll();

public slots:
    void refresh(const CuData &v);

protected slots:
    void slotItemEntered(QTreeWidgetItem*,int);
    void openUrl();


signals:
    void newAlarmDetected();
    void message(const QString& msg);
    void mouseOverItem(bool);
    void silence();
    void ack();
    void enableAlarm();
    void disableAlarm();


protected:
    void mouseMoveEvent(QMouseEvent *event);
    void contextMenuEvent(QContextMenuEvent *);

private:
    int m_pri_to_int(const QString& pri) const;
    int m_status_to_int(const QString& stat);
    void m_remove_stale_items(const QSet<QString> stale_names, const QString& src);
    void m_insert_item(const QString &src, const QStringList &list, int index);
    QString m_gen_error_item(const CuData& da) const;
    AlarmTreeWidgetItem *m_findItem(const QString& alarm, const QString& handler) const;

    bool _filter_enabled;
    QHash<QString, AlarmTreeWidgetItem*> mItemHash;
    QHash<QString, QSet<QString> >mSrcAlarmHash;
    QMap<QString, int> m_src_err_map;
    QString m_make_key(const QString& alarm, const QString& handler) const;

    Type m_type;
};



















#endif



