#include <AlarmTreeWidgetItem.h>
#include <QStringList>
#include <QDateTime>
#include <QTreeWidget>
#include <QtDebug>
#include <elettracolors.h>
#include <iostream>
#include <cumacros.h>

AlarmTreeWidgetItem::AlarmTreeWidgetItem(QTreeWidget *parent, QStringList &strings)
    : QTreeWidgetItem(parent, strings)
{
    m_init(strings);
}

AlarmTreeWidgetItem::AlarmTreeWidgetItem(QStringList &strings)
    : QTreeWidgetItem(strings)
{

    m_init(strings);
}

void AlarmTreeWidgetItem::m_init(QStringList& strings)
{
    _hide = false;

    if(strings.size() < 10)
    {
        for(int j = 0; j < 10 - strings.size(); j++)
            strings << "-";
    }

    setColor(strings.at(Status)/*, strings.at(SilencedMin).toInt()*/);

    update(strings);

    mHasMutedIcon = false;
    mNullIcon  = QIcon();
    mMutedIcon = QIcon(":icons/audio-volume-muted.png");
}

AlarmTreeWidgetItem::~AlarmTreeWidgetItem()
{

}

void AlarmTreeWidgetItem::setNew(bool isNew)
{
  QFont f = font(Alarm);
    f.setBold(isNew);
    setFont(Alarm, f);
}

void AlarmTreeWidgetItem::setColor(const QString& colorField)
{
    if(colorField.contains("ALARM"))
    {
        for(int j = 0; j < columnCount(); j++)
            setBackground(j, QColor(255,115,115));
    }
    else if(colorField.contains("WARNING"))
    {
        for(int j = 0; j < columnCount(); j++)
            setBackground(j, QBrush(EColor(Elettra::yellow)));
    }
    else
    {
        for(int j = 0; j < columnCount(); j++)
            setBackground(j, QBrush(EColor(Elettra::white)));
    }

}

void AlarmTreeWidgetItem::update(const QStringList &fields)
{
    QString colorField = fields.at(Status);
    QString levelField = fields.at(Level);
    int silenced = fields.at(SilencedMin).toInt();
    bool isNew = fields.last().toUpper() == "NEW";
    for(int i = 0; i < fields.size() && i < columnCount(); i++)
    {
        bool ok;
        QString field = fields.at(i);
        if(i == AlarmTreeWidgetItem::Device) {QRegularExpression re("[tango://]*(.*):\\d+/(.*)");
            QRegularExpressionMatch ma = re.match(field);
            if(ma.capturedTexts().size() > 2) {
                field = "[" + ma.capturedTexts().at(1).section('.', 0, 0) + "] " + ma.capturedTexts()[2];
            }
        }
        if(i == 0 && field.toUInt(&ok) && ok)
            field = QDateTime::fromTime_t(field.toUInt()).toString();
        if(text(i) != field)
            setText(i, field);
        if(colorField.contains("ALARM") || colorField.contains("ERROR")) {
            int l = colorField.contains("ERROR") ? 130 : 140;
            int d =  colorField.contains("ERROR") ? l * 1.4 : l * 0.85;
            if(levelField == "highest")
                setBackground(i, QColor("Red").lighter(i != Status ? l * 0.85 : d * 0.95));
            else if(levelField == "high")
                setBackground(i, QColor("Red").lighter(i != Status ? l : d));
            else if(levelField == "medium")
                setBackground(i, QColor("OrangeRed").lighter(i != Status ? l : d));
            else if(levelField == "low")
                setBackground(i, QColor("Orange").lighter(i != Status ? l : d));
            else if(levelField == "lowest")
                setBackground(i, QColor("yellow").lighter(i != Status ? l : d));
        }
        else
        {
                setBackground(i, QBrush(EColor(Elettra::white)));
        }

    }

    QFont f = font(Alarm);
    f.setBold(isNew);
    setFont(Alarm, f);

    if(silenced > 0 && !mHasMutedIcon)
    {
        setIcon(SilencedMin, mMutedIcon);
        QColor c = background(SilencedMin).color();
        c.setAlpha(100);
        setBackground(SilencedMin, c);
        mHasMutedIcon = true;
    }
    else if(mHasMutedIcon && silenced <= 0)
    {
        mHasMutedIcon = false;
        setIcon(SilencedMin, mNullIcon);
    }
}



