#include "alarminfow.h"
#include <cumbiapool.h>
#include <cucontrolsfactorypool.h>
#include <quwatcher.h>
#include <qustringlist.h>
#include <qustring.h>
#include <QHeaderView>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonValue>
#include <QJsonObject>

AlarmInfoW::AlarmInfoW(QWidget *parent, CumbiaPool *cu_pool, const CuControlsFactoryPool &fpoo) : QTreeView(parent)
{
    QuWatcher *iw = new QuWatcher(this, cu_pool, fpoo);
    iw->setObjectName("alarm_info_watcher");
    connect(iw, SIGNAL(newData(CuData)), this, SLOT(update(CuData)));
    QStandardItemModel *m = new QStandardItemModel(this);
    setHeaderHidden(true);
    setModel(m);
    setMouseTracking(true); // for hand cursor over urls
    setAlternatingRowColors(true);
    header()->setSectionResizeMode(QHeaderView::ResizeToContents);
    connect(this, SIGNAL(clicked(QModelIndex)), this, SLOT(onItemClicked(QModelIndex)));
    connect(this, SIGNAL(entered(QModelIndex)), this, SLOT(onItemEntered(QModelIndex)));
}

void AlarmInfoW::update(const CuData &da) {
    model()->removeRows(0, model()->rowCount());
    setDisabled(da["err"].toBool());
    setToolTip(da["msg"].toString().c_str());
    QStandardItemModel *m = qobject_cast<QStandardItemModel *>(model());
    if(!da["err"].toBool()) {
        QuStringList l(da["value"]);
        foreach(QString s, l) {
            QString t = m_line_break(s.section('=', 1), 120);
            // break into lines if too long
            QStandardItem *si1 = new QStandardItem(s.section('=', 0, 0));
            QStandardItem *si2 = new QStandardItem(t);
            si2->setToolTip(si2->text());
            if(si1->text() == "url" && si2->text().length() > 0) {
                si2->setForeground(QColor(Qt::blue));
                QFont f = si2->font();
                f.setUnderline(true);
                si2->setFont(f);
            }
            m->appendRow(QList<QStandardItem *>() << si1 << si2);
        }
    }/*
#if CUMBIA_VERSION < 0x010203
        m->appendRow(QList<QStandardItem *>() << new QStandardItem("cumbia version required") <<  new QStandardItem(">= 1.2.3"));
#endif*/
}

void AlarmInfoW::activate(const QString&dev, const QString& arg) {
    QTreeView::setVisible(!dev.isEmpty());
    if(dev.isEmpty()) {
        printf("AlarmInfoW.activate: unsetSource %s\n", qstoc(findChild<QuWatcher *>()->source()));
        findChild<QuWatcher *>()->unsetSource();
    }
    else {
        printf("AlarmInfoW.activate: %s\n", qstoc(QString("%1->GetAlarmInfo(%2,)").arg(dev).arg(arg)));
        findChild<QuWatcher *>()->setSource(QString("%1->GetAlarmInfo(%2,)").arg(dev).arg(arg));
    }
}

void AlarmInfoW::onItemClicked(const QModelIndex &mi) {
    QString t = mi.data().toString();
    QModelIndex sibling = model()->index(mi.row(), 0);
    if(sibling.data() == "url" && !t.isEmpty())
        emit urlClicked(t);
}

void AlarmInfoW::onItemEntered(const QModelIndex &mi) {
    QString t = mi.data().toString();
    QModelIndex sibling = model()->index(mi.row(), mi.column() - 1);
    setCursor(QCursor(sibling.data() == "url" && t.length() > 0 ?  Qt::PointingHandCursor : Qt::ArrowCursor));
}

QString AlarmInfoW::m_line_break(const QString &s, int linelen) const
{
    QString s1;
    QJsonDocument jd = QJsonDocument::fromJson(s.toLatin1());
    if(jd.isNull()) {
        s1 = s;
        if(s.contains("||") || s.contains("&&")) {
            s1.replace("||", "||\n").replace("&&", "&&\n");
        }
    }
    else {
        if(jd.isObject()) {
            QJsonObject jo = jd.object();
            foreach(const QString& k, jo.keys()) {
                QJsonValue val = jo[k];
                if(val.isDouble()) s1 += QString("%1: %2\n").arg(k).arg(val.toDouble());
                else if(val.isBool())  s1 += QString("%1: %2\n").arg(k).arg(val.toBool() ? "TRUE" : "FALSE");
                else if(val.isString()) s1 += QString("%1: \"%2\"\n").arg(k).arg(val.toString());
            }
        }
    }
    QString out;
    foreach(QString line, s1.split("\n")) {
        int i = 0;
        while(i + linelen  < line.length()) {
            line.insert(i + linelen, "↵\n");
            i+= linelen + 2;
        }
        out += line + "\n";
    }
    for(int i = out.length() - 1; i >=0 && out[i] == '\n'; i--)
        out.remove(i, 1);
    return out;
}
